@@ -17,10 +17,11 @@ def tracking(func):
     @wraps(func)
     def wrapper(*args, **kwargs):
         self = args[0]
-        if time.time() - self._last_tracking > tracking_interval:
-            self._publish_tracking()
-        if time.time() - self._last_heartbeat > heartbeat_interval:
-            self._publish_heartbeat()
+        if self._kind == 'central':
+            if time.time() - self._last_tracking > tracking_interval:
+                self._publish_tracking()
+            if time.time() - self._last_heartbeat > heartbeat_interval:
+                self._publish_heartbeat()
         return func(*args, **kwargs)
     return wrapper
 
@@ -41,7 +42,10 @@ class SmappeeMqtt(threading.Thread):
         return f'servicelocation/{self._service_location.service_location_uuid}'
 
     def _on_connect(self, client, userdata, flags, rc):
-        self._client.subscribe(topic=f'{self.topic_prefix}/#')
+        if self._kind == 'local':
+            self._client.subscribe(topic='#')
+        else:
+            self._client.subscribe(topic=f'{self.topic_prefix}/#')
 
     def _publish_tracking(self):
         # turn OFF current tracking and restore
@@ -107,6 +111,8 @@ class SmappeeMqtt(threading.Thread):
             elif message.topic == f'{self.topic_prefix}/config':
                 config_details = json.loads(message.payload)
                 self._service_location.firmware_version = config_details.get('firmwareVersion')
+                self._service_location._service_location_uuid = config_details.get('serviceLocationUuid')
+                self._service_location._service_location_id = config_details.get('serviceLocationId')
             elif message.topic == f'{self.topic_prefix}/sensorConfig':
                 pass
             elif message.topic == f'{self.topic_prefix}/homeControlConfig':
@@ -193,7 +199,7 @@ class SmappeeMqtt(threading.Thread):
                 elif message.topic.endswith('/property/chargingstate'):
                     pass
 
-                elif discovery:
+                elif config['MQTT']['discovery']:
                     print(message.topic, message.payload)
 
             # specific HASS.io topics
@@ -229,8 +235,9 @@ class SmappeeMqtt(threading.Thread):
 
     def start(self):
         self._client = mqtt.Client(client_id=self._get_client_id())
-        self._client.username_pw_set(username=self._service_location.service_location_uuid,
-                                     password=self._service_location.service_location_uuid)
+        if self._kind == 'central':
+            self._client.username_pw_set(username=self._service_location.service_location_uuid,
+                                         password=self._service_location.service_location_uuid)
         self._client.on_connect = lambda client, userdata, flags, rc: self._on_connect(client, userdata, flags, rc)
         self._client.on_message = lambda client, userdata, message: self._on_message(client, userdata, message)
         self._client.on_disconnect = lambda client, userdata, rc: self._on_disconnect(client, userdata, rc)
