@@ -1,17 +1,21 @@
-from .api import SmappeeApi
 from .servicelocation import SmappeeServiceLocation
 
 
 class Smappee(object):
 
-    def __init__(self, api):
+    def __init__(self, api, serialnumber=None):
         """
         :param api:
+        :param serialNumber:
         """
 
         # shared api instance
         self.smappee_api = api
 
+        # serialnumber (LOCAL env only)
+        self._serialnumber = serialnumber
+        self._local_polling = serialnumber is not None
+
         # service locations accessible from user
         self._service_locations = {}
 
@@ -31,6 +35,19 @@ class Smappee(object):
                 # Add sl object
                 self.service_locations[service_location.get('serviceLocationId')] = sl
 
+    def load_local_service_location(self):
+        # Create service location object
+        sl = SmappeeServiceLocation(device_serial_number=self._serialnumber,
+                                    smappee_api=self.smappee_api,
+                                    local_polling=self._local_polling)
+
+        # Add sl object
+        self.service_locations[sl.service_location_id] = sl
+
+    @property
+    def local_polling(self):
+        return self._local_polling
+
     @property
     def service_locations(self):
         return self._service_locations
